unit Custom;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Buttons,
  StdCtrls, Grids, ExtCtrls, PlusMemo;

type
  TDlgCustomKeywords = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    GroupBox1: TGroupBox;
    edDelimiters: TEdit;
    GroupBox2: TGroupBox;
    StrGKeywords: TStringGrid;
    GroupBox3: TGroupBox;
    ChkBMatchCase: TCheckBox;
    ChkBWholeWords: TCheckBox;
    ChkBHighlight: TCheckBox;
    ChkBAltFont: TCheckBox;
    ChkBItalic: TCheckBox;
    ChkBBold: TCheckBox;
    ChkBUnderline: TCheckBox;
    ChkBStrikeOut: TCheckBox;
    BtnRemove: TButton;
    LblHint: TLabel;
    procedure FormShow(Sender: TObject);
    procedure StrGKeywordsSelectCell(Sender: TObject; Col, Row: Longint;
      var CanSelect: Boolean);
    procedure StrGKeywordsSetEditText(Sender: TObject; ACol, ARow: Longint;
      const Value: String);
    procedure ChkBxClick(Sender: TObject);
    procedure edDelimitersChange(Sender: TObject);
    procedure BtnRemoveClick(Sender: TObject);
    procedure StrGKeywordsEnter(Sender: TObject);
    procedure StrGKeywordsExit(Sender: TObject);
    procedure StrGKeywordsGetEditText(Sender: TObject; ACol, ARow: Longint;
      var Value: OpenString);
    procedure StrGKeywordsKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
    inSelect: Boolean;
    Modified: Boolean;
    woList: TList;
  public
    { Public declarations }
    {class function EditKeywordList(KeywordList: TKeywordList): Boolean;}
  end;

var
  DlgCustomKeywords: TDlgCustomKeywords;

function EditKeywordList(KeywordList: TKeywordList): Boolean;

implementation
uses Messages;
{$R *.DFM}

type pWordInfo = ^TKeywordInfo;

function EditKeywordList(KeywordList: TKeywordList): Boolean;
  var delm: string[255];  i: Integer; c: Char;
      pk: pWordInfo;
  begin
  with TDlgCustomKeywords.Create(Application) do
    begin
    delm:= '';
    for c:= #1 to #255 do if c in KeywordList.Delimiters then delm:= delm+c;
    edDelimiters.Text:= delm;
    woList:= TList.Create;
    StrGKeywords.RowCount:= KeywordList.Count+1;

    for i:= 0 to KeywordList.Count-1 do
      begin
      woList.Add(KeywordList.Objects[i]);
      StrGKeywords.Cells[0, i]:= KeywordList[i]
      end;
    new(pk);
    if KeywordList.Count>0 then pk^:= pWordInfo(KeywordList.Objects[KeywordList.Count-1])^
    else with pk^ do
           begin
           Options:= [];
           Style:= [];
           ContextNumber:= 1;
           Backgnd:= 0; Foregnd:= 0
           end;
    woList.Add(pk);

    if (ShowModal=mrOk) and Modified then
      with Keywordlist do
        begin
        for i:= 0 to Count-1 do Objects[i]:= nil;
        Clear;
        BeginUpdate;
        for i:= 0 to StrGKeywords.RowCount-2 do
          AddObject(StrGKeywords.Cells[0,i], woList[i]);
        EndUpdate;
        delm:= edDelimiters.Text;
        Delimiters:= [];
        for i:= 1 to Length(delm) do Delimiters:= Delimiters + [delm[i]];
        Result:= True
      end
    else Result:= False;
    Free
    end
  end;

procedure TDlgCustomKeywords.FormShow(Sender: TObject);
var dummy: Boolean;
begin
  StrGKeywordsSelectCell(Self, 0, StrGKeywords.Row, dummy)
end;

procedure TDlgCustomKeywords.StrGKeywordsSelectCell(Sender: TObject; Col,
  Row: Longint; var CanSelect: Boolean);
begin
CanSelect:= True;
inSelect:= True;   { avoid OnClick updating info }
with pWordInfo(woList[Row])^ do
  begin
  ChkBMatchCase.Checked:= woMatchCase in Options;
  ChkBWholeWords.Checked:= woWholeWordsOnly in Options;
  ChkBBold.Checked:= fsBold in Style;
  ChkBItalic.Checked:= fsItalic in Style;
  ChkBUnderline.Checked:= fsUnderline in Style;
  ChkBStrikeOut.Checked:= fsStrikeOut in Style;
  ChkBHighlight.Checked:= fsHighlight in Style;
  ChkBAltFont.Checked:= fsAltFont in Style
  end;

inSelect:= False
end;

procedure TDlgCustomKeywords.StrGKeywordsSetEditText(Sender: TObject; ACol,
  ARow: Longint; const Value: String);
begin
Modified:= True;
LblHint.Show;
end;

procedure TDlgCustomKeywords.ChkBxClick(Sender: TObject);
begin
if not inSelect then
  with pWordInfo(woList[StrGKeywords.Row])^ do
    begin
    Modified:= True;
    if ChkBMatchCase.Checked then Options:= [woMatchCase]
                             else Options:= [];
    if ChkBWholeWords.Checked then Options:= Options + [woWholeWordsOnly];
    if ChkBBold.Checked then Style:= [fsBold]
                        else Style:= [];
    if ChkBItalic.Checked then Style:= Style + [fsItalic];
    if ChkBUnderline.Checked then Style:= Style + [fsUnderline];
    if ChkBStrikeOut.Checked then Style:= Style + [fsStrikeOut];
    if ChkBHighlight.Checked then Style:= Style + [fsHighlight];
    if ChkBAltFont.Checked then Style:= Style + [fsAltFont];
    end;
end;

procedure TDlgCustomKeywords.edDelimitersChange(Sender: TObject);
begin
Modified:= True
end;

procedure TDlgCustomKeywords.BtnRemoveClick(Sender: TObject);
var i: Integer;  dummy: Boolean;
var pk: pWordInfo;
begin
with StrGKeywords do
  if Row<RowCount-1 then
    begin
    pk:= woList[Row];
    Dispose(pk);
    for i:= Row to RowCount-2 do
        begin
        Rows[i]:= Rows[i+1];
        woList[i]:= woList[i+1];
        end;
    RowCount:= RowCount-1;
    woList.Count:= woList.Count-1;
    StrGKeywordsSelectCell(Self, 0, Row, dummy)
    end
end;

procedure TDlgCustomKeywords.StrGKeywordsEnter(Sender: TObject);
begin
LblHint.Show
end;

procedure TDlgCustomKeywords.StrGKeywordsExit(Sender: TObject);
begin
LblHint.Hide
end;

procedure TDlgCustomKeywords.StrGKeywordsGetEditText(Sender: TObject; ACol,
  ARow: Longint; var Value: OpenString);
begin
{LblHint.Hide }
end;

procedure TDlgCustomKeywords.StrGKeywordsKeyPress(Sender: TObject; var Key: Char);
var pk: pWordInfo;
begin
with StrGKeywords do
  if Row=RowCount-1 then
    begin
    RowCount:= RowCount+1;
    new(pk);
    woList.Add(pk);
    pk^:= pWordInfo(woList[Row])^
    end;

end;

end.
